@@ -34,10 +34,11 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
+#include <libempathy/empathy-client-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
-#include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-request-util.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-contact-list-view.h"
@@ -112,9 +113,6 @@ static const GtkTargetEntry drag_types_source[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
-static GdkAtom drag_atoms_dest[G_N_ELEMENTS (drag_types_dest)];
-static GdkAtom drag_atoms_source[G_N_ELEMENTS (drag_types_source)];
-
 enum {
 	DRAG_CONTACT_RECEIVED,
 	LAST_SIGNAL
@@ -385,7 +383,7 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 					 GtkSelectionData  *selection)
 {
 	EmpathyContactListViewPriv *priv;
-	TpAccountManager           *account_manager;
+	EmpathyClientFactory       *factory;
 	TpConnection               *connection = NULL;
 	TpAccount                  *account = NULL;
 	DndGetContactData          *data;
@@ -396,7 +394,6 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 	const gchar   *contact_id = NULL;
 	gchar         *new_group = NULL;
 	gchar         *old_group = NULL;
-	gboolean       success = TRUE;
 	gboolean       new_group_is_fake, old_group_is_fake = TRUE;
 
 	priv = GET_PRIV (view);
@@ -427,12 +424,14 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 		return FALSE;
 	}
 
-	account_manager = tp_account_manager_dup ();
+	factory = empathy_client_factory_dup ();
 	strv = g_strsplit (sel_data, ":", 2);
 	if (g_strv_length (strv) == 2) {
 		account_id = strv[0];
 		contact_id = strv[1];
-		account = tp_account_manager_ensure_account (account_manager, account_id);
+		account = tp_simple_client_factory_ensure_account (
+				TP_SIMPLE_CLIENT_FACTORY (factory),
+				account_id, NULL, NULL);
 	}
 	if (account) {
 		connection = tp_account_get_connection (account);
@@ -440,10 +439,9 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 
 	if (!connection) {
 		DEBUG ("Failed to get connection for account '%s'", account_id);
-		success = FALSE;
 		g_free (new_group);
 		g_free (old_group);
-		g_object_unref (account_manager);
+		g_object_unref (factory);
 		return FALSE;
 	}
 
@@ -459,7 +457,7 @@ contact_list_view_contact_drag_received (GtkWidget         *view,
 						data, (GDestroyNotify) contact_list_view_dnd_get_contact_free,
 						G_OBJECT (view));
 	g_strfreev (strv);
-	g_object_unref (account_manager);
+	g_object_unref (factory);
 
 	return TRUE;
 }
@@ -702,14 +700,14 @@ contact_list_view_drag_begin (GtkWidget      *widget,
 
 	priv = GET_PRIV (widget);
 
-	GTK_WIDGET_CLASS (empathy_contact_list_view_parent_class)->drag_begin (widget,
-									      context);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return;
 	}
 
+	GTK_WIDGET_CLASS (empathy_contact_list_view_parent_class)->drag_begin (widget,
+									      context);
+
 	path = gtk_tree_model_get_path (model, &iter);
 	priv->drag_row = gtk_tree_row_reference_new (model, path);
 	gtk_tree_path_free (path);
@@ -763,7 +761,8 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 	str = g_strconcat (account_id, ":", contact_id, NULL);
 
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
-		gtk_selection_data_set (selection, drag_atoms_source[info], 8,
+		gtk_selection_data_set (selection,
+					gdk_atom_intern ("text/contact-id", FALSE), 8,
 					(guchar *) str, strlen (str) + 1);
 	}
 
@@ -906,8 +905,8 @@ contact_list_view_row_activated (GtkTreeView       *view,
 
 	if (contact) {
 		DEBUG ("Starting a chat");
-		empathy_dispatcher_chat_with_contact (contact,
-			gtk_get_current_event_time ());
+		empathy_chat_with_contact (contact,
+			empathy_get_current_action_time ());
 		g_object_unref (contact);
 	}
 }
@@ -1287,19 +1286,15 @@ contact_list_view_search_key_navigation_cb (GtkWidget *search,
 					    GdkEvent *event,
 					    EmpathyContactListView *view)
 {
-	GdkEventKey *eventkey = ((GdkEventKey *) event);
+	GdkEvent *new_event;
 	gboolean ret = FALSE;
 
-	if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down) {
-		GdkEvent *new_event;
+	new_event = gdk_event_copy (event);
+	gtk_widget_grab_focus (GTK_WIDGET (view));
+	ret = gtk_widget_event (GTK_WIDGET (view), new_event);
+	gtk_widget_grab_focus (search);
 
-		new_event = gdk_event_copy (event);
-		gtk_widget_grab_focus (GTK_WIDGET (view));
-		ret = gtk_widget_event (GTK_WIDGET (view), new_event);
-		gtk_widget_grab_focus (search);
-
-		gdk_event_free (new_event);
-	}
+	gdk_event_free (new_event);
 
 	return ret;
 }
@@ -1508,7 +1503,6 @@ contact_list_view_constructed (GObject *object)
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	GtkCellRenderer            *cell;
 	GtkTreeViewColumn          *col;
-	guint                       i;
 
 	priv->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (
 			GTK_TREE_MODEL (priv->store), NULL));
@@ -1640,17 +1634,6 @@ contact_list_view_constructed (GObject *object)
 
 	/* Actually add the column now we have added all cell renderers */
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view), col);
-
-	/* Drag & Drop. */
-	for (i = 0; i < G_N_ELEMENTS (drag_types_dest); ++i) {
-		drag_atoms_dest[i] = gdk_atom_intern (drag_types_dest[i].target,
-						      FALSE);
-	}
-
-	for (i = 0; i < G_N_ELEMENTS (drag_types_source); ++i) {
-		drag_atoms_source[i] = gdk_atom_intern (drag_types_source[i].target,
-							FALSE);
-	}
 }
 
 static void
@@ -1886,7 +1869,6 @@ empathy_contact_list_view_new (EmpathyContactListStore        *store,
 EmpathyContact *
 empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1894,8 +1876,6 @@ empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return NULL;
@@ -1911,7 +1891,6 @@ empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 EmpathyContactListFlags
 empathy_contact_list_view_get_flags (EmpathyContactListView *view)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1919,8 +1898,6 @@ empathy_contact_list_view_get_flags (EmpathyContactListView *view)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), 0);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return 0;
@@ -1937,7 +1914,6 @@ gchar *
 empathy_contact_list_view_get_selected_group (EmpathyContactListView *view,
 					      gboolean *is_fake_group)
 {
-	EmpathyContactListViewPriv *priv;
 	GtkTreeSelection          *selection;
 	GtkTreeIter                iter;
 	GtkTreeModel              *model;
@@ -1947,8 +1923,6 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view,
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
-	priv = GET_PRIV (view);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		return NULL;
